package org.greenstone.server;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.util.ArrayList;
import javax.swing.*;
import javax.swing.border.*;

import org.greenstone.gsdl3.util.GlobalProperties;

import org.apache.log4j.*;
/**
 * This class is to start or restart the library server
 * 
 */
public class ServerControl extends JFrame {

    static Logger logger = Logger.getLogger(org.greenstone.server.ServerControl.class.getName());
 
    /** The dimension of the frame */
    static final private Dimension FRAME_SIZE = new Dimension(350, 250);

    Color bg_color = Color.white;    
    
    /** some components we need to refer to later */
    private JLabel info_label;
    private JButton enter_button;
    private JMenu fMenu;
    private Server server;
    private JFrame thisframe;
    
    public ServerControl(Server server) {
	super(Server.dictionary.get("ServerControl.Frame_Title"));
 
	this.server = server;
        thisframe = this;

	setSize(FRAME_SIZE);
	setDefaultCloseOperation(EXIT_ON_CLOSE);
	Dimension screen_size = Toolkit.getDefaultToolkit().getScreenSize();
	setLocation((screen_size.width - FRAME_SIZE.width) / 2,
		    (screen_size.height - FRAME_SIZE.height) / 2);
	setBackground(Color.white);
        addWindowListener(new MyAdapter()); 
   
        JPanel title_panel = new JPanel();
	title_panel.setLayout(new BorderLayout());

        JLabel title_label = new JLabel();
        String title = Server.dictionary.get("ServerControl.Title");           
        title_label.setText(stringToHTML(title)); 
        title_label.setOpaque(false);	
	title_label.setHorizontalAlignment(SwingConstants.CENTER);
        title_label.setFont(new Font("SansSerif",Font.PLAIN,18));
       

        JLabel version_label = new JLabel();
        String version = Server.dictionary.get("ServerControl.Version", new String [] {GlobalProperties.getProperty(StaticStrings.GSDL3_VERSION_PROPERTY)}).toLowerCase();
         
        version_label.setText(stringToHTML(version)); 
        version_label.setOpaque(false);	
	version_label.setHorizontalAlignment(SwingConstants.CENTER);
        version_label.setFont(new Font("SansSerif",Font.PLAIN,14));
        
        title_panel.add(title_label,BorderLayout.CENTER);
        title_panel.add(version_label,BorderLayout.SOUTH);
	title_panel.setBackground(bg_color);	
	title_panel.setBorder(BorderFactory.createEmptyBorder(5,10,5,10));

   	info_label = new JLabel();
        info_label.setOpaque(false);
        info_label.setHorizontalAlignment(SwingConstants.LEFT);
        info_label.setVerticalAlignment(SwingConstants.CENTER);
        info_label.setFont(new Font("SansSerif",Font.PLAIN,14));
	info_label.setBorder(BorderFactory.createEmptyBorder(5,10,5,10));	        
		
        JPanel button_panel = new JPanel(new FlowLayout(FlowLayout.CENTER));
	enter_button = new JButton();
	enter_button.setEnabled(false);
	enter_button.addActionListener(new EnterButtonListener());
        enter_button.setText(Server.dictionary.get("ServerControl.EnterLibrary"));
 	button_panel.add(enter_button);
	button_panel.setBackground(bg_color);	
	button_panel.setBorder(BorderFactory.createEmptyBorder(5,10,5,10));

	getContentPane().setLayout(new BorderLayout());
	getContentPane().add(title_panel,BorderLayout.NORTH);
	getContentPane().add(info_label,BorderLayout.CENTER);
	getContentPane().add(button_panel,BorderLayout.SOUTH);
	getContentPane().setBackground(bg_color);
    	setJMenuBar(createMenu());
	setVisible(true);
    }

    protected JMenuBar createMenu() {
	JMenuBar menuBar = new JMenuBar();
         fMenu = new JMenu(Server.dictionary.get("ServerControl.Menu.File"));
	JMenuItem iConf = new JMenuItem(Server.dictionary.get("ServerControl.Menu.Settings"));
	iConf.setBackground(Color.white);
	iConf.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    ServerSettings serverSetting = new ServerSettings(server);
		}
	    });
	
	JMenuItem iExit = new JMenuItem(Server.dictionary.get("ServerControl.Menu.Exit"));
	iExit.setBackground(Color.white);
	iExit.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent ae) {
		    Thread runInThread = new Thread(new Runnable(){
			    public void run(){
				server.stop();
			    
			    }
			},"stop server");
		    try{
                        runInThread.start(); 
		
		    }
		    catch(Exception e){
			logger.error(e);
		    }    
		    thisframe.dispose();
		}
	    });
	fMenu.add(iConf);
	fMenu.add(iExit);
	fMenu.setEnabled(false);
	menuBar.add(fMenu);
	menuBar.setBorder(new BevelBorder(BevelBorder.RAISED));
	return menuBar;
    }

    public void updateControl(){
      	switch (server.getServerState()){
	case Server.SERVER_STARTED:
	    {
		info_label.setText(stringToHTML(Server.dictionary.get("ServerControl.Help_EnterLibrary")));
		enter_button.setEnabled(true);
		fMenu.setEnabled(true);
                break;
	    }
	case Server.SERVER_START_FAILED:
	    {
		enter_button.setText(stringToHTML(Server.dictionary.get("ServerControl.StartServer")));
		enter_button.setEnabled(true);
		fMenu.setEnabled(true);
		break;	
	    }
	case Server.BROWSER_LAUNCHED:
	    {
		info_label.setText(stringToHTML(Server.dictionary.get("ServerControl.Help_RestartLibrary")));
		enter_button.setText(stringToHTML(Server.dictionary.get("ServerControl.RestartLibrary")));
		thisframe.setState(Frame.ICONIFIED);
		enter_button.setEnabled(true);
                fMenu.setEnabled(true);
		break;
	    }
	case Server.BROWSER_LAUNCH_FAILED:
	    {
		enter_button.setText(stringToHTML(Server.dictionary.get("ServerControl.EnterLibrary")));
		enter_button.setEnabled(true);
		fMenu.setEnabled(true);
                 break;	
	    }
        default:{
	    enter_button.setEnabled(false);
	    fMenu.setEnabled(false);
	}
	}
    }
           
   
    public void displayMessage(String message){
	info_label.setText(stringToHTML("<br>"+message));           
    }
    
    private class MyAdapter extends WindowAdapter{   
	public void windowClosing(WindowEvent env){
            Thread runInThread = new Thread(new Runnable(){
			    public void run(){
				server.stop();
				
			    }
		},"stop server");
	    try{
		runInThread.start(); 
	    }
	    catch(Exception e){
		logger.error(e);
	    }  
	    thisframe.dispose();
	}
    } 

    private String stringToHTML(String s){
	return "<html><body>"+s+"</body></html>";	
    }
    
    private class EnterButtonListener
	implements ActionListener {
	
	public void actionPerformed(ActionEvent ev) {
	    switch (server.getServerState()){
	    case Server.SERVER_STARTED: case Server.BROWSER_LAUNCH_FAILED:
		{
		 Thread runInThread = new Thread(new Runnable(){
			    public void run(){
			    server.launchBrowser();
			    }
			},"launch browser");
		    
		    runInThread.start(); 
                   break;     
		}
	    case Server.SERVER_START_FAILED:
		{
		    Thread runInThread = new Thread(new Runnable(){
			    public void run(){
				server.start();
			    }
			},"start server");
		    runInThread.start();
		    break;
		}  	
	    case Server.BROWSER_LAUNCHED:
		{
		    Thread runInThread = new Thread(new Runnable(){
			    public void run(){
				server.restart();
				  
			    }
			},"restart server");
		    runInThread.start();
		}  
	    }   
	  
	}
    }
}
