<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:java="http://xml.apache.org/xslt/java"
  xmlns:util="xalan://org.greenstone.gsdl3.util.XSLTUtil"
  extension-element-prefixes="java util"
  exclude-result-prefixes="java util">
  
  <!-- some global parameters - these are set by whoever is invoking the transformation -->
  <xsl:param name="interface_name"/>
  <xsl:param name="library_name"/>

  <!-- the main page layout template is here -->
  <xsl:template match="page">
    <html>
      <xsl:call-template name="pageHead"/>
      <body class="bgimage">
	<div id="page">
	   <xsl:apply-templates select="pageResponse"/>
	   <xsl:call-template name="greenstoneFooter"/>                	   
	</div>
      </body>
    </html>
  </xsl:template>

  
  <!-- put a space in the title in case the actual value is missing - mozilla will not display a page with no title-->
  <xsl:template name="pageHead">
    <head>
      <title>
	<xsl:call-template name="pageTitle"/><xsl:text> </xsl:text>
      </title>
      <xsl:call-template name="globalStyle"/>
      <xsl:call-template name="pageStyle"/>
    </head>
  </xsl:template>

  <!-- any global style stuff should go in here -->
  <xsl:template name="globalStyle">
    <link rel="stylesheet" href="interfaces/classic/style/classic.css" type="text/css"/>
     <link rel="stylesheet" href="interfaces/classic/style/classic2.css" type="text/css"/>
  </xsl:template>
  
  <xsl:template name="greenstonePageBanner">
     <div id="banner">
        <div class="collectimage">
            <img src="interfaces/default/images/gsdlhead.gif"><xsl:attribute name="alt"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'gsdl')"/></xsl:attribute>
            </img>
	  </div>
      </div>
      <div class="bannerextra"></div>
      <div id="content">
        <xsl:call-template name="dividerBar">
            <xsl:with-param name="text" select="'select a collection'"/> 
        </xsl:call-template>
      </div>			
   </xsl:template>

  <xsl:template name="genericPageBanner">
    <xsl:param name='text'/>
    <xsl:variable name="lang" select="/page/@lang"/>
    <center>
      <table width="100%">
	<tr><td align="left"></td>
	  <td align="right">
	    <table>
	      <tr><td class="gsbutton">
		  <a class="gsbutton" href="{$library_name}?a=p&amp;sa=home">
		    <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'home_b')"/>
		  </a>
		</td></tr></table>
	  </td>
	</tr>
      </table>
    </center>
    <xsl:call-template name="dividerBar"><xsl:with-param name='text' select="$text"/></xsl:call-template>
  </xsl:template>
  
  <xsl:template name="standardPageBanner">
    <xsl:param name="collName"/>
    <xsl:param name="pageType"/>
    <xsl:param name="clTop"/>
    <xsl:variable name="this-element" select="/page/pageResponse/collection"/>
     <div id="bannar">
      <div class="pageinfo"> 
          <xsl:call-template name="top-buttons">
	    <xsl:with-param name="collName" select="$collName"/>
	    <xsl:with-param name="pageType" select="$pageType"/>
	  </xsl:call-template>
          <xsl:if test="not(/page/pageRequest/@action='d')">
  	<xsl:variable name="text">
	  <xsl:choose>
	    <xsl:when test="$pageType='browse'">
	      <xsl:value-of select="/page/pageResponse/collection/serviceList/service[@name='ClassifierBrowse']/classifierList/classifier[@name=$clTop]/displayItem[@name='name']"/>
	    </xsl:when>
	    <xsl:otherwise>
	      <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, concat($pageType, '_t'))"/>
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:variable>
        <p class="bannertitle"><xsl:value-of select="$text"/></p>	 
      </xsl:if>
     </div>

     <div class="collectimage">
       <a href="{$library_name}?a=p&amp;sa=about&amp;c={$collName}">
	    <xsl:choose>
	      <xsl:when test="$this-element/displayItem[@name='icon']">
		<img border="0">
		  <xsl:attribute name="src">
		    <xsl:value-of select="$this-element/metadataList/metadata[@name='httpPath']"/>/images/<xsl:value-of select="$this-element/displayItem[@name='icon']"/>
		  </xsl:attribute>	
		  <xsl:attribute name="alt">
		    <xsl:value-of select="$this-element/displayItem[@name='name']"/>
		  </xsl:attribute>
		  <xsl:attribute name="title">
		    <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'aboutpage')"/>
		  </xsl:attribute>
		</img>
	      </xsl:when>
	      <xsl:otherwise>
		<b><xsl:value-of select="$this-element/displayItem[@name='name']"/>hello</b>
	      </xsl:otherwise>
	    </xsl:choose>
	  </a>
       </div>
      </div>
    <div class="bannerextra"></div>	
  </xsl:template>
  
  <xsl:template name="top-buttons">
    <xsl:param name="collName"/>
    <xsl:param name="pageType"/>
    <p class="bannerlinks">
      	  <a class="navlink" href="{$library_name}?a=p&amp;sa=home"><xsl:attribute name="title"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'home_tip')"/></xsl:attribute>
	    <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'home_b')"/>
	  </a>
	<!-- <xsl:choose>
	<xsl:when test="$pageType='help'">
             <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'help_b')"/>
        </xsl:when>
	<xsl:otherwise>
	  <a class="navlink" href="{$library_name}?a=p&amp;sa=help&amp;c={$collName}"><xsl:attribute name="title"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'help_tip')"/></xsl:attribute>
	<xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'help_b')"/>
          </a>
       </xsl:otherwise>
      </xsl:choose> -->
       <xsl:choose>
	  <xsl:when test="$pageType='pref'">
	      <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'pref_b')"/>
	   </xsl:when>
	  <xsl:otherwise>
	    <a class="navlink" href="{$library_name}?a=p&amp;sa=pref&amp;c={$collName}"><xsl:attribute name="title"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'pref_tip')"/></xsl:attribute>
		<xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'pref_b')"/>
	    </a>
	   </xsl:otherwise>
	</xsl:choose>      
    </p>
  </xsl:template>
  
  <xsl:template name="navigationBar">
    <xsl:param name="collName"/>
    <xsl:param name="clTop"/>
    <xsl:variable name="this-element" select="/page/pageResponse/collection"/>
    <xsl:variable name="this-service" select="/page/pageResponse/service/@name"/>
    <xsl:variable name="classifiers" select="$this-element/serviceList/service[@name='ClassifierBrowse']/classifierList"/>
     <div class="navbar">
       <p class="navbar">
	 <xsl:if test="$this-element/serviceList/service[@type='query']">
	   <xsl:call-template name="navbar-search"><xsl:with-param name="collName" select="$collName"/></xsl:call-template>
	 </xsl:if>
	
	<xsl:for-each select="$classifiers/classifier">
	  <xsl:apply-templates select="." mode="navbar">
	    <xsl:with-param name="collName" select="$collName"/>
	    <xsl:with-param name="clTop" select="$clTop"/>
	  </xsl:apply-templates>
	</xsl:for-each>
	
	<xsl:if test="$this-element/serviceList/service[@name='PhindApplet']">
	  <xsl:call-template name="navbar-phind"><xsl:with-param name="collName" select="$collName"/></xsl:call-template>
	</xsl:if>
       </p>
      </div> 
  </xsl:template>

  <xsl:template name="navbar-search">
    <xsl:param name="collName"/>
    <xsl:choose>
      <xsl:when test="/page/pageRequest/@action='q'">	  
	   <span class="narspace">search</span>
      </xsl:when>
      <xsl:otherwise>
	<xsl:variable name="service">
	  <xsl:choose>
	    <xsl:when  test="not(/page/pageRequest/paramList/param[@name='ct'])">TextQuery</xsl:when>
	    <xsl:when test="/page/pageRequest/paramList/param[@name='ct']/@value='0'">TextQuery</xsl:when>
	    <xsl:when test="/page/pageRequest/paramList/param[@name='qt']/@value='1'"><xsl:choose><xsl:when test="/page/pageRequest/paramList/param[@name='qfm']/@value='1'">AdvancedFieldQuery</xsl:when><xsl:otherwise>FieldQuery</xsl:otherwise></xsl:choose></xsl:when>
	    <xsl:otherwise>TextQuery</xsl:otherwise>
	  </xsl:choose>
	</xsl:variable>
	<a class="navlink" href="{$library_name}?a=q&amp;rt=d&amp;s={$service}&amp;c={$collName}"><xsl:attribute name='title'><xsl:value-of select="/page/pageResponse/collection/service[@name=$service]/displayItem[@name='description']"/></xsl:attribute>search</a>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="navbar-phind">
    <xsl:param name="collName"/>
    <xsl:choose>
      <xsl:when test="/page/pageRequest/paramList/param[@name='s']/@value='PhindApplet'">
	    <span class="narspace">phrases</span>
      </xsl:when>
      <xsl:otherwise>
	 <a class="navlink" href="{$library_name}?a=a&amp;rt=d&amp;s=PhindApplet&amp;c={$collName}"><xsl:if test="/page/pageResponse/collection/service[@name='PhindApplet']/displayItem[@name='description']"><xsl:attribute name='title'><xsl:value-of select="/page/pageResponse/collection/service[@name='PhindApplet']/displayItem[@name='description']"/></xsl:attribute></xsl:if>phrases</a>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="classifier" mode="navbar">
    <xsl:param name="collName"/>
    <xsl:param name="clTop"/>
    <xsl:choose>
      <xsl:when test="/page/pageRequest/@action='b' and $clTop=@name">
	 <span class="narspace"><xsl:value-of select="displayItem[@name='name']"/></span>
      </xsl:when>
      <xsl:otherwise>
	<a class="navlink" href="{$library_name}?a=b&amp;rt=r&amp;s=ClassifierBrowse&amp;c={$collName}&amp;cl={@name}"><xsl:attribute name='title'><xsl:value-of select="displayItem[@name='description']"/></xsl:attribute><xsl:value-of select="displayItem[@name='name']"/></a>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="dividerBar">   
    <xsl:param name='text'/>
      <xsl:choose>
      <xsl:when test="$text">
	 <div class="divbar"><p class="navbar"><xsl:value-of select="$text"/></p></div>
      </xsl:when>
      <xsl:otherwise>
	 <div class="divbar"><xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text></div>
      </xsl:otherwise>
    </xsl:choose> 
  </xsl:template>
  
  <xsl:template name="nzdlPageBanner">
       <div><img src="interfaces/nzdl/images/nzdl2gr.gif" width="457" height="181" ><xsl:attribute name="alt"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'nzdl')"/></xsl:attribute></img></div>
       <!-- end of page banner -->  
  </xsl:template>

  <xsl:template name="greenstoneFooter">
    <div class="divbar"><p class="navbar">powered by greenstone3</p>
    </div>
  </xsl:template>
  
  <xsl:template name="imagegreenstone">
    <img src="interfaces/nzdl/images/gsdl.gif" width="140" height="77" border="0" hspace='0'><xsl:attribute name='alt'><xsl:call-template name='textimagegreenstone'/></xsl:attribute></img>
  </xsl:template>
  
  <!-- Display the appropriate image, depending on the node type -->
  <xsl:template match="documentNode" mode="displayNodeIcon">
    
    <!-- Root node: book icon (open or closed) -->
    <xsl:choose>
      <xsl:when test="@nodeType='root'">
	<xsl:choose>
	  <xsl:when test="documentNode">
	    <xsl:call-template name="openbookimg">
	      <xsl:with-param name="title">Close this book</xsl:with-param>
	    </xsl:call-template>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:call-template name="closedbookimg">
	      <xsl:with-param name="title">Open this document and view contents</xsl:with-param>
	    </xsl:call-template>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when>
      
      <!-- Internal node: folder icon (open or closed) -->
      <xsl:when test="@nodeType='internal'">
	<xsl:choose>
	  <xsl:when test="documentNode">
	    <xsl:call-template name="openfolderimg">
	      <xsl:with-param name="title">Close this folder</xsl:with-param>
	    </xsl:call-template>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:call-template name="closedfolderimg">
	      <xsl:with-param name="title">Open this folder and view contents</xsl:with-param>
	    </xsl:call-template>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when>
      
      <!-- Leaf node: page icon, and this is the default -->
      <xsl:otherwise>
	<xsl:call-template name="textpageimg">
	  <xsl:with-param name="title">View the document</xsl:with-param>
	</xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <xsl:template name="openbookimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="28" height="23" 
      src="interfaces/default/images/openbook.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="closedbookimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="18" height="11"  
      src="interfaces/default/images/book.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="openfolderimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="23" height="15"  
      src="interfaces/default/images/openfldr.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="closedfolderimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="23" height="15" 
      src="interfaces/default/images/clsdfldr.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="textpageimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="16" height="21"
      src="interfaces/default/images/itext.gif"
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="bookshelfimg">
    <xsl:param name="alt"/>
    <xsl:param name="title"/>
    <img border="0" width="20" height="16" 
      src="interfaces/default/images/bshelf.gif"  
      alt="{$alt}" title="{$title}"/>
  </xsl:template>

  <xsl:template name="iconpdf">
    <xsl:param name="alt">PDF</xsl:param>
    <xsl:param name="title"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'texticonpdf')"/></xsl:param>
    <img border="0" width='26' height='26' 
      src='interfaces/default/images/ipdf.gif' 
      alt='{$alt}' title='{$title}'/>
  </xsl:template>
    
  <xsl:template name="icondoc">
    <xsl:param name="alt">Word</xsl:param>
    <xsl:param name="title"><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'texticonmsword')"/></xsl:param>
    <img border="0" width='26' height='26' 
      src='interfaces/default/images/imsword.gif' 
      alt='{$alt}' title='{$title}'/>
  </xsl:template>

</xsl:stylesheet>  
